require 'zip_search'
require 'pg_search'

module ZipSearch
  module ActsAsLocation
    extend ActiveSupport::Concern
    include PgSearch

    included do
    end

    module ClassMethods
      def acts_as_location(title=:location, options={})
        pg_search_scope :search_by_location,
                         against: [:zip, :county, :city, :state],
                         using: {tsearch: {prefix: true}}
        pg_search_scope :search_by_state, against: :state,
                         using: {tsearch: {prefix: true}}
        pg_search_scope :search_by_city, against: :city,
                         using: {tsearch: {prefix: true}}
        pg_search_scope :search_by_county, :against => :county,
                         using: {tsearch: {prefix: true}}
        pg_search_scope :search_by_zip, against: :zip

        geocoded_by :to_sentence
        reverse_geocoded_by :latitude, :longitude do |obj, results|
          if geo = results.first
            obj.street_address ||= obj.street_address
            obj.zip    ||= geo.postal_code
            obj.county ||= geo.county if geo.respond_to? :county
            obj.city   ||= geo.city
            obj.state  ||= geo.state
          end
        end
        after_validation :geocode, if: :should_geocode?
        after_validation :reverse_geocode, if: :should_reverse_geocode?

        validates_presence_of :zs_association_name

        include ActsAsLocation::LocalInstanceMethods
      end

      def location_fields; %i( street_address zip county city state ) end

      def params(extra_params=[])
        extra_params + [ :zs_association_name, :title, :latitude, :longitude,
                         :street_address, :zip, :county, :city, :state ]
      end

      def nested_params(extra_params=[])
        [:id, :_destroy] + params(extra_params)
      end
    end

    module LocalInstanceMethods

      def location_fields; self.class.location_fields end
      def location; Hash[ location_fields.map{|lf| [ lf, send(lf) ] } ] end
      def location_blank?; location.values.reject(&:blank?).empty? end
      def location_missing_any?; location.values.select(&:blank?).any? end

      def to_sentence
        return 'Location unavailable' if location_blank?
        location_fields.map{|lf| send(lf) }.reject(&:blank?).join(', ')
      end

      def has_coordinates?; !latitude.nil? && !longitude.nil? end
      def coordinates; [latitude, longitude] end

      def should_geocode?; !has_coordinates? && !location_blank? end
      def should_reverse_geocode?; has_coordinates? && location_missing_any? end
      
    end

  end
end

