require 'zip_search'

module ZipSearch
  class Engine < ::Rails::Engine
    isolate_namespace ZipSearch

    require 'zip_search/controller_helpers'
    require 'zip_search/model_helpers'
    require 'zip_search/view_helpers'
    require 'zip_search/simple_form_helper'

    initializer 'zip_search.controller_helpers' do
      ActionController::Base.send :include, ControllerHelpers
    end
    initializer 'zip_search.model_helpers' do
      ActiveRecord::Base.send :include, ModelHelpers
    end
    initializer 'zip_search.view_helpers' do
      ActionView::Base.send :include, ViewHelpers
      if defined? SimpleForm
        ActionView::Base.send :include, SimpleFormHelper
        SimpleFormHelper::METHODS_TO_EXPORT.each do |m|
          SimpleForm::FormBuilder.send :define_method, m do |*args, &block|
            template.send m, self, *args, &block
          end
        end
      end
    end
    initializer :append_migrations do |app|
      unless app.root.to_s == root.to_s
        config.paths['db/migrate'].expanded.each do |expanded_path|
          app.config.paths['db/migrate'] << expanded_path
        end
      end
    end

  end
end
